"""
    NoteType Constants

    All note MUST include their TYPE so the consumer can
    change behavior depending on the type given

    When Creating a child class, a new type MUST be created
    and appended at the end of the file no matter how similar
    it is with an existing note type.
"""
from enum import Enum


class NoteType(Enum):
    BASE_NOTE = 'BaseNote'

    ANIME_NOTE = 'AnimNote'



from datetime import datetime

from meiki.NoteType import NoteType

"""
    The SimpleNote only implements the
    base metadata of the note.
    
    The body/content should be implemented by the
    child classes to prevent polymorphic behavior
    where an audio note is read as text file
"""


class BaseNote:

    def __init__(self):
        self._title = None
        self._created_on = datetime.now()
        self._updated_on = datetime.now()
        self._note_type = NoteType.BASE_NOTE

    @property
    def title(self) -> str:
        return self._title

    @title.setter
    def title(self, title: str) -> None:
        self._title = title

    @property
    def note_type(self) -> NoteType:
        return self._note_type

    @note_type.setter
    def note_type(self, note_type: NoteType) -> None:
        self._note_type = note_type



from BaseNote import BaseNote
from NoteType import NoteType


class AnimeNote(BaseNote):

    def __init__(self):
        super().__init__()
        self._note_type = NoteType.ANIME_NOTE

        self._japanese_title = None
        self._english_title = None
        self._romaji_title = None
        self._rating = 0

    @property
    def japanese_title(self) -> str:
        return self._japanese_title

    @japanese_title.setter
    def japanese_title(self, title: str) -> None:
        self._japanese_title = title

        # Only set the Japanese title if it's blank (no english title)
        if self._title is None:
            self._title = title

    @property
    def romaji_title(self) -> str:
        return self._romaji_title

    @romaji_title.setter
    def romaji_title(self, title) -> None:
        self._romaji_title = title

        # Romaji should be our last resort for title
        if self._title is None and self._japanese_title is None:
            self._title = title

    @property
    def english_title(self) -> str:
        return self._english_title

    @english_title.setter
    def english_title(self, title) -> None:
        self._english_title = title

        # Always override the title to english
        self._title = title
